@@ -664,7 +664,7 @@ init_from_xml(LassoNode *node, xmlNode *xmlnode)
 /*****************************************************************************/
 
 static void
-instance_init(LassoNameRegistration *name_registration)
+instance_init(LassoNameRegistration *name_registration, G_GNUC_UNUSED void *unused)
 {
 	name_registration->oldNameIdentifier = NULL;
 }
