@@ -58,17 +58,7 @@ typedef struct _LassoProfilePrivate LassoProfilePrivate;
  * @LASSO_REQUEST_TYPE_NAME_REGISTRATION: Name Registration
  * @LASSO_REQUEST_TYPE_NAME_IDENTIFIER_MAPPING: Name Identifier Mapping
  * @LASSO_REQUEST_TYPE_LECP: Liberty-Enabled Client / Proxy
- * @LASSO_REQUEST_TYPE_DISCO_QUERY: ID-WSF 1.0 Discovery Query request
- * @LASSO_REQUEST_TYPE_DISCO_MODIFY: ID-WSF 1.0 Discovery Modify Request
- * @LASSO_REQUEST_TYPE_DST_QUERY: ID-WSF 1.0 Data Service Template Query request
- * @LASSO_REQUEST_TYPE_DST_MODIFY: ID-WSF 1.0 Data Service Temaplte Modify request
- * @LASSO_REQUEST_TYPE_SASL_REQUEST: ID-WSF 1.0 Authentication request
  * @LASSO_REQUEST_TYPE_NAME_ID_MANAGEMENT: SAML 2.0 NameID Management request
- * @LASSO_REQUEST_TYPE_IDWSF2_DISCO_SVCMD_REGISTER: ID-WSF 2.0 Discovery Service Metadata Register
- * request
- * @LASSO_REQUEST_TYPE_IDWSF2_DISCO_SVCMD_ASSOCIATION_ADD: ID-WSF 2.0 Discovery Service Metadata
- * Add Association request
- * @LASSO_REQUEST_TYPE_IDWSF2_DISCO_QUERY: ID-WSF 2.0 Discovery Query request
  *
  * Request types (known for SOAP endpoints)
  */
@@ -80,15 +70,7 @@ typedef enum {
 	LASSO_REQUEST_TYPE_NAME_REGISTRATION = 4,
 	LASSO_REQUEST_TYPE_NAME_IDENTIFIER_MAPPING = 5,
 	LASSO_REQUEST_TYPE_LECP = 6,
-	LASSO_REQUEST_TYPE_DISCO_QUERY = 7,
-	LASSO_REQUEST_TYPE_DISCO_MODIFY = 8,
-	LASSO_REQUEST_TYPE_DST_QUERY = 9,
-	LASSO_REQUEST_TYPE_DST_MODIFY = 10,
-	LASSO_REQUEST_TYPE_SASL_REQUEST = 11,
 	LASSO_REQUEST_TYPE_NAME_ID_MANAGEMENT = 12,
-	LASSO_REQUEST_TYPE_IDWSF2_DISCO_SVCMD_REGISTER = 13,
-	LASSO_REQUEST_TYPE_IDWSF2_DISCO_SVCMD_ASSOCIATION_ADD = 14,
-	LASSO_REQUEST_TYPE_IDWSF2_DISCO_QUERY = 15
 } LassoRequestType;
 
 /**
@@ -211,6 +193,15 @@ LASSO_EXPORT LassoProfileSignatureVerifyHint lasso_profile_get_signature_verify_
 LASSO_EXPORT LassoProviderRole lasso_profile_sso_role_with(LassoProfile *profile,
 		const char *remote_provider_id);
 LASSO_EXPORT lasso_error_t lasso_profile_get_signature_status(LassoProfile *profile);
+LASSO_EXPORT char* lasso_profile_get_issuer(const char *message);
+LASSO_EXPORT char* lasso_profile_get_in_response_to(const char *message);
+
+LASSO_EXPORT char* lasso_profile_get_message_id(LassoProfile *profile);
+LASSO_EXPORT void lasso_profile_set_message_id(LassoProfile *profile, const char *message_id);
+
+LASSO_EXPORT LassoNode* lasso_profile_get_idp_list(LassoProfile *profile);
+LASSO_EXPORT void lasso_profile_set_idp_list(LassoProfile *profile, const LassoNode *idp_list);
+
 
 #ifdef __cplusplus
 }
