@@ -68,7 +68,7 @@ static struct XmlSnippet schema_snippets[] = {
 
 
 static void
-class_init(LassoLibAuthenticationStatementClass *klass)
+class_init(LassoLibAuthenticationStatementClass *klass, void *unused G_GNUC_UNUSED)
 {
 	LassoNodeClass *nclass = LASSO_NODE_CLASS(klass);
 
