@@ -18,8 +18,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "../private.h"
@@ -57,9 +56,11 @@
 
 static struct XmlSnippet schema_snippets[] = {
 	{ "EncryptedData", SNIPPET_XMLNODE,
-		G_STRUCT_OFFSET(LassoSaml2EncryptedElement, EncryptedData), NULL, NULL, NULL},
+		G_STRUCT_OFFSET(LassoSaml2EncryptedElement, EncryptedData), NULL, LASSO_XMLENC_PREFIX,
+		LASSO_XMLENC_HREF},
 	{ "EncryptedKey", SNIPPET_LIST_XMLNODES,
-		G_STRUCT_OFFSET(LassoSaml2EncryptedElement, EncryptedKey), NULL, NULL, NULL},
+		G_STRUCT_OFFSET(LassoSaml2EncryptedElement, EncryptedKey), NULL, LASSO_XMLENC_PREFIX,
+		LASSO_XMLENC_HREF},
 	{ "NameID", SNIPPET_NODE | SNIPPET_LASSO_DUMP,
 		G_STRUCT_OFFSET(LassoSaml2EncryptedElement, original_data), NULL, NULL, NULL},
 	{NULL, 0, 0, NULL, NULL, NULL}
@@ -73,7 +74,7 @@ static LassoNodeClass *parent_class = NULL;
 /*****************************************************************************/
 
 static void
-class_init(LassoSaml2EncryptedElementClass *klass)
+class_init(LassoSaml2EncryptedElementClass *klass, void *unused G_GNUC_UNUSED)
 {
 	LassoNodeClass *nclass = LASSO_NODE_CLASS(klass);
 
