@@ -46,10 +46,6 @@
 #include <libxml/parserInternals.h>
 #include <libxml/xmlIO.h>
 
-#include <openssl/pem.h>
-#include <openssl/sha.h>
-#include <openssl/engine.h>
-#include <openssl/hmac.h>
 #include <openssl/evp.h>
 
 #include <xmlsec/base64.h>
@@ -60,6 +56,7 @@
 #include <xmlsec/errors.h>
 #include <xmlsec/openssl/x509.h>
 #include <xmlsec/openssl/crypto.h>
+#include <xmlsec/openssl/evp.h>
 
 #include <zlib.h>
 
@@ -73,6 +70,7 @@
 #include <stdarg.h>
 #include <ctype.h>
 #include "../lasso_config.h"
+#include "config.h"
 
 /**
  * SECTION:saml2_utils
@@ -481,22 +479,23 @@ char*
 lasso_query_sign(char *query, LassoSignatureContext context)
 {
 	char *digest = NULL; /* 160 bit buffer */
-	RSA *rsa = NULL;
-	DSA *dsa = NULL;
 	unsigned char *sigret = NULL;
-	unsigned int siglen = 0;
+	size_t siglen = 0;
 	xmlChar *b64_sigret = NULL, *e_b64_sigret = NULL;
 	char *new_query = NULL, *s_new_query = NULL;
 	int status = 0;
 	const xmlChar *algo_href = NULL;
-	char *hmac_key;
+	unsigned char *hmac_key;
 	size_t hmac_key_length;
-	const EVP_MD *md = NULL;
 	xmlSecKey *key;
 	xmlSecKeyData *key_data;
-	unsigned int sigret_size = 0;
 	LassoSignatureMethod sign_method;
-        lasso_error_t rc = 0;
+	lasso_error_t rc = 0;
+
+	const EVP_MD *md = NULL;
+	EVP_MD_CTX *evp_md_ctx = NULL;
+	EVP_PKEY *pkey = NULL;
+	EVP_PKEY *hmac_pkey = NULL;
 
 	g_return_val_if_fail(query != NULL, NULL);
 	g_return_val_if_fail(lasso_ok_signature_method(context.signature_method), NULL);
@@ -546,98 +545,42 @@ lasso_query_sign(char *query, LassoSignatureContext context)
 		xmlFree(BAD_CAST t);
 	}
 
-	/* build buffer digest */
+	/* define the digest algorithm */
 	switch (sign_method) {
 		case LASSO_SIGNATURE_METHOD_RSA_SHA1:
 		case LASSO_SIGNATURE_METHOD_DSA_SHA1:
-			digest = lasso_sha1(new_query);
-			break;
-		case LASSO_SIGNATURE_METHOD_RSA_SHA256:
-			digest = lasso_sha256(new_query);
-			break;
-		case LASSO_SIGNATURE_METHOD_RSA_SHA384:
-			digest = lasso_sha384(new_query);
-			break;
-		case LASSO_SIGNATURE_METHOD_RSA_SHA512:
-			digest = lasso_sha512(new_query);
-		default:
-			break;
-	}
-	switch (sign_method) {
-		case LASSO_SIGNATURE_METHOD_RSA_SHA1:
-		case LASSO_SIGNATURE_METHOD_DSA_SHA1:
-		case LASSO_SIGNATURE_METHOD_RSA_SHA256:
-		case LASSO_SIGNATURE_METHOD_RSA_SHA384:
-		case LASSO_SIGNATURE_METHOD_RSA_SHA512:
-			if (digest == NULL) {
-				message(G_LOG_LEVEL_CRITICAL, "Failed to build the buffer digest");
-				goto done;
-			}
-		default:
-			break;
-	}
-	/* extract the OpenSSL key */
-	switch (sign_method) {
-		case LASSO_SIGNATURE_METHOD_RSA_SHA1:
-		case LASSO_SIGNATURE_METHOD_RSA_SHA256:
-		case LASSO_SIGNATURE_METHOD_RSA_SHA384:
-		case LASSO_SIGNATURE_METHOD_RSA_SHA512:
-			rsa = xmlSecOpenSSLKeyDataRsaGetRsa(key_data);
-			g_assert(rsa);
-			/* alloc memory for sigret */
-			sigret_size = RSA_size(rsa);
-			break;
-		case LASSO_SIGNATURE_METHOD_DSA_SHA1:
-			dsa = xmlSecOpenSSLKeyDataDsaGetDsa(key_data);
-			g_assert(dsa);
-			/* alloc memory for sigret */
-			sigret_size = DSA_size(dsa);
-			break;
 		case LASSO_SIGNATURE_METHOD_HMAC_SHA1:
 			md = EVP_sha1();
-			sigret_size = EVP_MD_size(md);
-			break;
-		case LASSO_SIGNATURE_METHOD_HMAC_SHA256:
-			md = EVP_sha256();
-			sigret_size = EVP_MD_size(md);
-			break;
-		case LASSO_SIGNATURE_METHOD_HMAC_SHA384:
-			md = EVP_sha384();
-			sigret_size = EVP_MD_size(md);
-			break;
-		case LASSO_SIGNATURE_METHOD_HMAC_SHA512:
-			md = EVP_sha512();
-			sigret_size = EVP_MD_size(md);
-			break;
-		default:
-			g_assert_not_reached();
-	}
-	sigret = (unsigned char *)g_malloc (sigret_size);
-
-	switch (sign_method) {
-		case LASSO_SIGNATURE_METHOD_RSA_SHA1:
-			/* sign digest message */
-			status = RSA_sign(NID_sha1, (unsigned char*)digest, SHA_DIGEST_LENGTH, sigret,
-					&siglen, rsa);
 			break;
 		case LASSO_SIGNATURE_METHOD_RSA_SHA256:
-			/* sign digest message */
-			status = RSA_sign(NID_sha256, (unsigned char*)digest, SHA256_DIGEST_LENGTH, sigret,
-					&siglen, rsa);
+		case LASSO_SIGNATURE_METHOD_HMAC_SHA256:
+			md = EVP_sha256();
 			break;
 		case LASSO_SIGNATURE_METHOD_RSA_SHA384:
-			/* sign digest message */
-			status = RSA_sign(NID_sha384, (unsigned char*)digest, SHA384_DIGEST_LENGTH, sigret,
-					&siglen, rsa);
+		case LASSO_SIGNATURE_METHOD_HMAC_SHA384:
+			md = EVP_sha384();
 			break;
 		case LASSO_SIGNATURE_METHOD_RSA_SHA512:
-			/* sign digest message */
-			status = RSA_sign(NID_sha512, (unsigned char*)digest, SHA512_DIGEST_LENGTH, sigret,
-					&siglen, rsa);
+		case LASSO_SIGNATURE_METHOD_HMAC_SHA512:
+			md = EVP_sha512();
 			break;
+		case LASSO_SIGNATURE_METHOD_NONE:
+		case LASSO_SIGNATURE_METHOD_LAST:
+			g_assert_not_reached();
+	}
+
+	/* Get the signture key */
+	switch (sign_method) {
+		case LASSO_SIGNATURE_METHOD_RSA_SHA1:
+		case LASSO_SIGNATURE_METHOD_RSA_SHA256:
+		case LASSO_SIGNATURE_METHOD_RSA_SHA384:
+		case LASSO_SIGNATURE_METHOD_RSA_SHA512:
 		case LASSO_SIGNATURE_METHOD_DSA_SHA1:
-			status = DSA_sign(NID_sha1, (unsigned char*)digest, SHA_DIGEST_LENGTH, sigret,
-					&siglen, dsa);
+			pkey = xmlSecOpenSSLEvpKeyDataGetEvp(key_data);
+			if (! pkey) {
+				message(G_LOG_LEVEL_CRITICAL, "Failed to get assymetric key");
+				goto done;
+			}
 			break;
 		case LASSO_SIGNATURE_METHOD_HMAC_SHA1:
 		case LASSO_SIGNATURE_METHOD_HMAC_SHA256:
@@ -649,15 +592,42 @@ lasso_query_sign(char *query, LassoSignatureContext context)
 				goto done;
 			}
 			g_assert(hmac_key);
-
-			/* key should be at least 128 bits long */
-			if (hmac_key_length < 16) {
-				critical("HMAC key should be at least 128 bits long");
+			hmac_pkey = EVP_PKEY_new_mac_key(EVP_PKEY_HMAC, NULL, hmac_key, (int)hmac_key_length);
+			if (! hmac_key) {
+				message(G_LOG_LEVEL_CRITICAL, "EVP_PKEY_new_mac_key failed");
 				goto done;
 			}
+			pkey = hmac_pkey;
+			break;
+		case LASSO_SIGNATURE_METHOD_LAST:
+		case LASSO_SIGNATURE_METHOD_NONE:
+			g_assert_not_reached();
+	}
 
-			HMAC(md, hmac_key, hmac_key_length, (unsigned char *)new_query,
-					strlen(new_query), sigret, &siglen);
+	switch (sign_method) {
+		case LASSO_SIGNATURE_METHOD_RSA_SHA1:
+		case LASSO_SIGNATURE_METHOD_RSA_SHA256:
+		case LASSO_SIGNATURE_METHOD_RSA_SHA384:
+		case LASSO_SIGNATURE_METHOD_RSA_SHA512:
+		case LASSO_SIGNATURE_METHOD_DSA_SHA1:
+		case LASSO_SIGNATURE_METHOD_HMAC_SHA1:
+		case LASSO_SIGNATURE_METHOD_HMAC_SHA256:
+		case LASSO_SIGNATURE_METHOD_HMAC_SHA384:
+		case LASSO_SIGNATURE_METHOD_HMAC_SHA512:
+			evp_md_ctx = EVP_MD_CTX_create();
+			if (EVP_DigestSignInit(evp_md_ctx, NULL, md, NULL, pkey) <= 0) {
+				message(G_LOG_LEVEL_CRITICAL, "EVP_DigestSignInit failed");
+				goto done;
+			}
+			if (EVP_DigestSign(evp_md_ctx, NULL, &siglen, (unsigned char*)new_query, strlen(new_query)) <= 0) {
+				message(G_LOG_LEVEL_CRITICAL, "EVP_DigestSign failed");
+				goto done;
+			}
+			sigret = g_malloc(siglen);
+			if (EVP_DigestSign(evp_md_ctx, sigret, &siglen, (unsigned char*)new_query, strlen(new_query)) <= 0) {
+				message(G_LOG_LEVEL_CRITICAL, "EVP_DigestSign failed");
+				goto done;
+			}
 			status = 1;
 			break;
 		case LASSO_SIGNATURE_METHOD_LAST:
@@ -665,14 +635,13 @@ lasso_query_sign(char *query, LassoSignatureContext context)
 			g_assert_not_reached();
 	}
 
-	g_assert(siglen == sigret_size);
 
 	if (status == 0) {
 		goto done;
 	}
 
 	/* Base64 encode the signature value */
-	b64_sigret = xmlSecBase64Encode(sigret, sigret_size, 0);
+	b64_sigret = xmlSecBase64Encode(sigret, siglen, 0);
 	/* escape b64_sigret */
 	e_b64_sigret = lasso_xmlURIEscapeStr((xmlChar*)b64_sigret, NULL);
 
@@ -701,7 +670,15 @@ done:
 	lasso_release(sigret);
 	lasso_release_xml_string(b64_sigret);
 	lasso_release_xml_string(e_b64_sigret);
-
+	if (evp_md_ctx) {
+		EVP_MD_CTX_free(evp_md_ctx);
+		evp_md_ctx = NULL;
+	}
+	if (hmac_pkey) {
+		EVP_PKEY_free(hmac_pkey);
+		hmac_pkey = NULL;
+		pkey = NULL;
+	}
 	return s_new_query;
 }
 
@@ -728,153 +705,162 @@ lasso_assertion_encrypt(LassoSaml2Assertion *assertion, char *recipient)
 
 static lasso_error_t
 lasso_query_verify_helper(const char *signed_content, const char *b64_signature, const char *algorithm,
-		const xmlSecKey *key)
+		xmlSecKey *key)
 {
-	RSA *rsa = NULL;
-	DSA *dsa = NULL;
 	char *digest = NULL;
-	xmlSecByte *signature = NULL;
-	int key_size = 0;
+	char *signature = NULL;
+	unsigned int signature_len = 0;
 	unsigned char *hmac_key = NULL;
 	size_t hmac_key_length = 0;
 	const EVP_MD *md = NULL;
 	lasso_error_t rc = 0;
 	LassoSignatureMethod method = LASSO_SIGNATURE_METHOD_NONE;
-	size_t digest_size = 1;
-	int type = -1;
+	EVP_PKEY *hmac_pkey = NULL;
+	unsigned char *new_signature = NULL;
+	EVP_MD_CTX *evp_md_ctx = NULL;
 
 	if (lasso_strisequal(algorithm, (char*)xmlSecHrefRsaSha1)) {
 		goto_cleanup_if_fail_with_rc(key->value->id == xmlSecOpenSSLKeyDataRsaId,
 				LASSO_DS_ERROR_INVALID_SIGALG)
-		rsa = xmlSecOpenSSLKeyDataRsaGetRsa(key->value);
-		key_size = RSA_size(rsa);
 		method = LASSO_SIGNATURE_METHOD_RSA_SHA1;
-		digest_size = SHA_DIGEST_LENGTH;
-		type = NID_sha1;
 	} else if (lasso_strisequal(algorithm, (char*)xmlSecHrefDsaSha1)) {
 		goto_cleanup_if_fail_with_rc(key->value->id == xmlSecOpenSSLKeyDataDsaId, LASSO_DS_ERROR_INVALID_SIGALG);
-		dsa = xmlSecOpenSSLKeyDataDsaGetDsa(key->value);
-		key_size = DSA_size(dsa);
 		method = LASSO_SIGNATURE_METHOD_DSA_SHA1;
-		digest_size = SHA_DIGEST_LENGTH;
-		type = NID_sha1;
 	} else if (lasso_strisequal(algorithm, (char*)xmlSecHrefRsaSha256)) {
 		goto_cleanup_if_fail_with_rc(key->value->id == xmlSecOpenSSLKeyDataRsaId,
 				LASSO_DS_ERROR_INVALID_SIGALG)
-		rsa = xmlSecOpenSSLKeyDataRsaGetRsa(key->value);
-		key_size = RSA_size(rsa);
 		method = LASSO_SIGNATURE_METHOD_RSA_SHA256;
-		digest_size = SHA256_DIGEST_LENGTH;
-		type = NID_sha256;
 	} else if (lasso_strisequal(algorithm, (char*)xmlSecHrefRsaSha384)) {
 		goto_cleanup_if_fail_with_rc(key->value->id == xmlSecOpenSSLKeyDataRsaId,
 				LASSO_DS_ERROR_INVALID_SIGALG)
-		rsa = xmlSecOpenSSLKeyDataRsaGetRsa(key->value);
-		key_size = RSA_size(rsa);
 		method = LASSO_SIGNATURE_METHOD_RSA_SHA384;
-		digest_size = SHA384_DIGEST_LENGTH;
-		type = NID_sha384;
 	} else if (lasso_strisequal(algorithm, (char*)xmlSecHrefRsaSha512)) {
 		goto_cleanup_if_fail_with_rc(key->value->id == xmlSecOpenSSLKeyDataRsaId,
 				LASSO_DS_ERROR_INVALID_SIGALG)
-		rsa = xmlSecOpenSSLKeyDataRsaGetRsa(key->value);
-		key_size = RSA_size(rsa);
 		method = LASSO_SIGNATURE_METHOD_RSA_SHA512;
-		digest_size = SHA512_DIGEST_LENGTH;
-		type = NID_sha512;
 	} else if (lasso_strisequal(algorithm, (char*)xmlSecHrefHmacSha1)) {
 		lasso_check_good_rc(lasso_get_hmac_key(key, (void**)&hmac_key, &hmac_key_length));
-		md = EVP_sha1();
-		key_size = EVP_MD_size(md);
 		method = LASSO_SIGNATURE_METHOD_HMAC_SHA1;
 	} else if (lasso_strisequal(algorithm, (char*)xmlSecHrefHmacSha256)) {
 		lasso_check_good_rc(lasso_get_hmac_key(key, (void**)&hmac_key, &hmac_key_length));
-		md = EVP_sha256();
-		key_size = EVP_MD_size(md);
 		method = LASSO_SIGNATURE_METHOD_HMAC_SHA256;
 	} else if (lasso_strisequal(algorithm, (char*)xmlSecHrefHmacSha384)) {
 		lasso_check_good_rc(lasso_get_hmac_key(key, (void**)&hmac_key, &hmac_key_length));
-		md = EVP_sha384();
-		key_size = EVP_MD_size(md);
 		method = LASSO_SIGNATURE_METHOD_HMAC_SHA384;
 	} else if (lasso_strisequal(algorithm, (char*)xmlSecHrefHmacSha512)) {
 		lasso_check_good_rc(lasso_get_hmac_key(key, (void**)&hmac_key, &hmac_key_length));
-		md = EVP_sha512();
-		key_size = EVP_MD_size(md);
 		method = LASSO_SIGNATURE_METHOD_HMAC_SHA512;
 	} else {
 		goto_cleanup_with_rc(LASSO_DS_ERROR_INVALID_SIGALG);
 	}
 
-	/* is the signature algo allowed */
-	goto_cleanup_if_fail_with_rc(
-                lasso_allowed_signature_method(method),
-                LASSO_DS_ERROR_INVALID_SIGALG);
-
-	/* decode signature */
-	signature = g_malloc(key_size+1);
-	goto_cleanup_if_fail_with_rc(
-			xmlSecBase64Decode((xmlChar*)b64_signature, signature, key_size+1) != 0,
-			LASSO_DS_ERROR_INVALID_SIGNATURE);
-	/* digest */
+	/* define the digest algorithm */
 	switch (method) {
 		case LASSO_SIGNATURE_METHOD_RSA_SHA1:
 		case LASSO_SIGNATURE_METHOD_DSA_SHA1:
-			digest = lasso_sha1(signed_content);
+		case LASSO_SIGNATURE_METHOD_HMAC_SHA1:
+			md = EVP_sha1();
 			break;
 		case LASSO_SIGNATURE_METHOD_RSA_SHA256:
-			digest = lasso_sha256(signed_content);
+		case LASSO_SIGNATURE_METHOD_HMAC_SHA256:
+			md = EVP_sha256();
 			break;
 		case LASSO_SIGNATURE_METHOD_RSA_SHA384:
-			digest = lasso_sha384(signed_content);
+		case LASSO_SIGNATURE_METHOD_HMAC_SHA384:
+			md = EVP_sha384();
 			break;
 		case LASSO_SIGNATURE_METHOD_RSA_SHA512:
-			digest = lasso_sha512(signed_content);
-			break;
-		case LASSO_SIGNATURE_METHOD_HMAC_SHA1:
-		case LASSO_SIGNATURE_METHOD_HMAC_SHA256:
-		case LASSO_SIGNATURE_METHOD_HMAC_SHA384:
 		case LASSO_SIGNATURE_METHOD_HMAC_SHA512:
+			md = EVP_sha512();
 			break;
-		default:
+		case LASSO_SIGNATURE_METHOD_NONE:
+		case LASSO_SIGNATURE_METHOD_LAST:
 			g_assert_not_reached();
 	}
+
+	/* is the signature algo allowed */
+	goto_cleanup_if_fail_with_rc(
+			lasso_allowed_signature_method(method),
+			LASSO_DS_ERROR_INVALID_SIGALG);
+
+	/* decode signature */
+	goto_cleanup_if_fail_with_rc(
+			lasso_base64_decode(b64_signature, &signature, (int*)&signature_len),
+			LASSO_DS_ERROR_INVALID_SIGNATURE);
 	/* verify signature */
+	evp_md_ctx = EVP_MD_CTX_create();
+
 	switch (method) {
 		case LASSO_SIGNATURE_METHOD_RSA_SHA1:
 		case LASSO_SIGNATURE_METHOD_RSA_SHA256:
 		case LASSO_SIGNATURE_METHOD_RSA_SHA384:
 		case LASSO_SIGNATURE_METHOD_RSA_SHA512:
-			goto_cleanup_if_fail_with_rc(
-					RSA_verify(
-						type,
-						(unsigned char*)digest,
-						digest_size,
-						signature,
-						key_size, rsa) == 1,
-					LASSO_DS_ERROR_INVALID_SIGNATURE);
-			break;
 		case LASSO_SIGNATURE_METHOD_DSA_SHA1:
-			goto_cleanup_if_fail_with_rc(
-					DSA_verify(
-						type,
-						(unsigned char*)digest,
-						digest_size,
-						signature,
-						key_size, dsa) == 1,
-					LASSO_DS_ERROR_INVALID_SIGNATURE);
+			{
+				xmlSecKeyData *key_data = xmlSecKeyGetValue(key);
+				if (! key_data) {
+					message(G_LOG_LEVEL_CRITICAL, "Failed to get KeyData");
+					goto_cleanup_with_rc(LASSO_DS_ERROR_INVALID_SIGNATURE);
+				}
+				EVP_PKEY *pkey = xmlSecOpenSSLEvpKeyDataGetEvp(key_data);
+				if (! pkey) {
+					message(G_LOG_LEVEL_CRITICAL, "Failed to get assymetric key");
+					goto_cleanup_with_rc(LASSO_DS_ERROR_INVALID_SIGNATURE);
+				}
+				if (1 != EVP_DigestVerifyInit(evp_md_ctx, NULL, md, NULL, pkey)) {
+					message(G_LOG_LEVEL_CRITICAL, "EVP_DigestVerifyInit failed");
+					goto_cleanup_with_rc(LASSO_DS_ERROR_INVALID_SIGNATURE);
+				}
+				if (1 != EVP_DigestVerifyUpdate(evp_md_ctx, signed_content, strlen(signed_content))) {
+					message(G_LOG_LEVEL_CRITICAL, "EVP_DigestVerifyUpdate failed");
+					goto_cleanup_with_rc(LASSO_DS_ERROR_INVALID_SIGNATURE);
+				}
+				if(1 != EVP_DigestVerifyFinal(evp_md_ctx, (unsigned char*)signature, signature_len)) {
+					goto_cleanup_with_rc(LASSO_DS_ERROR_INVALID_SIGNATURE);
+				}
+			}
 			break;
 		case LASSO_SIGNATURE_METHOD_HMAC_SHA1:
 		case LASSO_SIGNATURE_METHOD_HMAC_SHA256:
 		case LASSO_SIGNATURE_METHOD_HMAC_SHA384:
 		case LASSO_SIGNATURE_METHOD_HMAC_SHA512:
-			digest = g_malloc(key_size);
-			HMAC(md, hmac_key, hmac_key_length, (unsigned char*)signed_content,
-				strlen(signed_content), (unsigned char*)digest, NULL);
+			{
+				unsigned char *hmac_key;
+				size_t hmac_key_length;
+				size_t new_signature_len;
 
-			goto_cleanup_if_fail_with_rc(lasso_crypto_memequal(digest, signature,
-						key_size),
-					LASSO_DS_ERROR_INVALID_SIGNATURE);
+				if ((rc = lasso_get_hmac_key(key, (void**)&hmac_key,
+											 &hmac_key_length))) {
+					message(G_LOG_LEVEL_CRITICAL, "Failed to get hmac key (%s)", lasso_strerror(rc));
+					goto_cleanup_with_rc(LASSO_DS_ERROR_INVALID_SIGNATURE);
+				}
+				g_assert(hmac_key);
+				hmac_pkey = EVP_PKEY_new_mac_key(EVP_PKEY_HMAC, NULL, hmac_key, (int)hmac_key_length);
+				if (! hmac_key) {
+					message(G_LOG_LEVEL_CRITICAL, "EVP_PKEY_new_mac_key failed");
+					goto_cleanup_with_rc(LASSO_DS_ERROR_INVALID_SIGNATURE);
+				}
+
+				if (EVP_DigestSignInit(evp_md_ctx, NULL, md, NULL, hmac_pkey) != 1) {
+					message(G_LOG_LEVEL_CRITICAL, "EVP_DigestSignInit failed");
+					goto_cleanup_with_rc(LASSO_DS_ERROR_INVALID_SIGNATURE);
+				}
+				if (EVP_DigestSign(evp_md_ctx, NULL, &new_signature_len, (unsigned char*)signed_content, strlen(signed_content)) != 1) {
+					message(G_LOG_LEVEL_CRITICAL, "EVP_DigestSign failed");
+					goto_cleanup_with_rc(LASSO_DS_ERROR_INVALID_SIGNATURE);
+				}
+				if (new_signature_len != signature_len) {
+					goto_cleanup_with_rc(LASSO_DS_ERROR_INVALID_SIGNATURE);
+				}
+				new_signature = g_malloc(new_signature_len);
+				if (EVP_DigestSign(evp_md_ctx, new_signature, &new_signature_len, (unsigned char*)signed_content, strlen(signed_content)) != 1) {
+					message(G_LOG_LEVEL_CRITICAL, "EVP_DigestSign failed");
+					goto_cleanup_with_rc(LASSO_DS_ERROR_INVALID_SIGNATURE);
+				}
+				if (CRYPTO_memcmp(signature, new_signature, signature_len) != 0) {
+					goto_cleanup_with_rc(LASSO_DS_ERROR_INVALID_SIGNATURE);
+				}
+			}
 			break;
 		case LASSO_SIGNATURE_METHOD_NONE:
 		case LASSO_SIGNATURE_METHOD_LAST:
@@ -882,7 +868,15 @@ lasso_query_verify_helper(const char *signed_content, const char *b64_signature,
 	}
 cleanup:
 	lasso_release_string(digest);
-	lasso_release_string(signature);
+	lasso_release_string(new_signature);
+	if (evp_md_ctx) {
+		EVP_MD_CTX_free(evp_md_ctx);
+		evp_md_ctx = NULL;
+	}
+	if (hmac_pkey) {
+		EVP_PKEY_free(hmac_pkey);
+		hmac_pkey = NULL;
+	}
 	return rc;
 
 }
@@ -899,7 +893,7 @@ cleanup:
  * a negative value if an error occurs during verification
  **/
 lasso_error_t
-lasso_query_verify_signature(const char *query, const xmlSecKey *sender_public_key)
+lasso_query_verify_signature(const char *query, xmlSecKey *sender_public_key)
 {
 	gchar **str_split = NULL;
 	char *b64_signature = NULL;
@@ -958,7 +952,7 @@ cleanup:
  * Return value: 0 if signature is validated, an error code otherwise.
  */
 int
-lasso_saml2_query_verify_signature(const char *query, const xmlSecKey *sender_public_key)
+lasso_saml2_query_verify_signature(const char *query, xmlSecKey *sender_public_key)
 {
 	char *b64_signature = NULL;
 	char *query_copy = NULL;
@@ -1364,17 +1358,19 @@ lasso_get_query_string_param_value(const char *qs, const char *param_key, const
 }
 
 unsigned char*
-lasso_inflate(unsigned char *input, size_t len)
+lasso_inflate(unsigned char *input, size_t len, size_t *outlen)
 {
 	z_stream zstr;
 	unsigned char *output;
 	int z_err;
 
+	*outlen = 0;
 	zstr.zalloc = NULL;
 	zstr.zfree = NULL;
 	zstr.opaque = NULL;
 
-	output = g_malloc(len*20);
+	// add one to account for the zero byte
+	output = g_malloc(len*20+1);
 	zstr.avail_in = len;
 	zstr.next_in = (unsigned char*)input;
 	zstr.total_in = 0;
@@ -1398,6 +1394,7 @@ lasso_inflate(unsigned char *input, size_t len)
 	}
 	output[zstr.total_out] = 0;
 	inflateEnd(&zstr);
+	*outlen = zstr.total_out;
 
 	return output;
 }
@@ -1406,36 +1403,32 @@ lasso_inflate(unsigned char *input, size_t len)
 gboolean
 lasso_node_init_from_deflated_query_part(LassoNode *node, char *deflate_string)
 {
-	int len;
-	xmlChar *b64_zre, *zre, *re;
-	xmlDoc *doc;
-	xmlNode *root;
+	xmlChar *buffer= NULL;
+	char *decoded = NULL;
+	int decoded_len = 0;
+	xmlChar *re = NULL;
+	size_t re_len = 0;
+	xmlDoc *doc = NULL;
+	xmlNode *root = NULL;
+	gboolean rc = TRUE;
 
-	b64_zre = (xmlChar*)xmlURIUnescapeString(deflate_string, 0, NULL);
-	len = strlen((char*)b64_zre);
-	zre = xmlMalloc(len*4);
-	len = xmlSecBase64Decode(b64_zre, zre, len*4);
-	xmlFree(b64_zre);
-	if (len == -1) {
-		message(G_LOG_LEVEL_CRITICAL, "Failed to base64-decode query");
-		xmlFree(zre);
-		return FALSE;
-	}
+	buffer = (xmlChar*)xmlURIUnescapeString(deflate_string, 0, NULL);
+	goto_cleanup_if_fail_with_rc(lasso_base64_decode((char*)buffer, &decoded, &decoded_len), FALSE);
 
-	re = lasso_inflate(zre, len);
-	xmlFree(zre);
-
-	if (! re)
-		return FALSE;
+	re = lasso_inflate((unsigned char*)decoded, decoded_len, &re_len);
+	goto_cleanup_if_fail_with_rc_with_warning(re != NULL, FALSE);
 
 	doc = lasso_xml_parse_memory((char*)re, strlen((char*)re));
-	lasso_release_string(re);
+	goto_cleanup_if_fail_with_rc_with_warning(doc != NULL, FALSE);
 
 	root = xmlDocGetRootElement(doc);
 	lasso_node_init_from_xml(node, root);
+cleanup:
+	lasso_release_xml_string(buffer);
+	lasso_release_string(decoded);
+	lasso_release_string(re);
 	lasso_release_doc(doc);
-
-	return TRUE;
+	return rc;
 }
 
 char*
@@ -1896,40 +1889,44 @@ lasso_xml_get_soap_content(xmlNode *root)
 LassoMessageFormat
 lasso_xml_parse_message(const char *message, LassoMessageFormat constraint, xmlDoc **doc_out, xmlNode **root_out)
 {
-	char *msg = NULL;
-	gboolean b64 = FALSE;
+	const char *msg = NULL;
+	int msg_len = 0;
+	char *base64_decoded_message = NULL;
 	LassoMessageFormat rc = LASSO_MESSAGE_FORMAT_UNKNOWN;
 	xmlDoc *doc = NULL;
 	xmlNode *root = NULL;
 	gboolean any = constraint == LASSO_MESSAGE_FORMAT_UNKNOWN;
 
-	msg = (char*)message;
-
 	/* BASE64 case */
 	if (any || constraint == LASSO_MESSAGE_FORMAT_BASE64) {
 		if (message[0] != 0 && is_base64(message)) {
-			msg = g_malloc(strlen(message));
-			rc = xmlSecBase64Decode((xmlChar*)message, (xmlChar*)msg, strlen(message));
-			if (rc >= 0) {
-				b64 = TRUE;
-			} else {
-				lasso_release(msg);
-				msg = (char*)message;
+			if (lasso_base64_decode(message, &base64_decoded_message, &msg_len)) {
+				rc = LASSO_MESSAGE_FORMAT_BASE64;
+				msg = base64_decoded_message;
 			}
 		}
 	}
 
+	if (! msg) {
+		msg = message;
+		msg_len = strlen(message);
+	}
+
 	/* XML case */
 	if (any || constraint == LASSO_MESSAGE_FORMAT_BASE64 ||
 		constraint == LASSO_MESSAGE_FORMAT_XML ||
 		constraint == LASSO_MESSAGE_FORMAT_SOAP) {
 		if (strchr(msg, '<')) {
-			doc = lasso_xml_parse_memory(msg, strlen(msg));
+			doc = lasso_xml_parse_memory(msg, msg_len);
 			if (doc == NULL) {
 				rc = LASSO_MESSAGE_FORMAT_UNKNOWN;
 				goto cleanup;
 			}
 			root = xmlDocGetRootElement(doc);
+			if (! root) {
+				rc = LASSO_MESSAGE_FORMAT_ERROR;
+				goto cleanup;
+			}
 
 			if (any || constraint == LASSO_MESSAGE_FORMAT_SOAP) {
 				gboolean is_soap = FALSE;
@@ -1938,26 +1935,20 @@ lasso_xml_parse_message(const char *message, LassoMessageFormat constraint, xmlD
 				if (is_soap) {
 					root = lasso_xml_get_soap_content(root);
 				}
-				if (! root) {
-					rc = LASSO_MESSAGE_FORMAT_ERROR;
-					goto cleanup;
-				}
 				if (is_soap) {
 					rc = LASSO_MESSAGE_FORMAT_SOAP;
 					goto cleanup;
 				}
-				if (b64) {
-					lasso_release(msg);
-					rc = LASSO_MESSAGE_FORMAT_BASE64;
+				if (rc == LASSO_MESSAGE_FORMAT_BASE64) {
 					goto cleanup;
 				}
 				rc = LASSO_MESSAGE_FORMAT_XML;
-				goto cleanup;
 			}
 		}
 	}
 
 cleanup:
+	lasso_release(base64_decoded_message);
 	if (doc_out) {
 		*doc_out = doc;
 		if (root_out) {
@@ -1965,7 +1956,6 @@ cleanup:
 		}
 	} else {
 		lasso_release_doc(doc);
-		lasso_release_xml_node(root);
 	}
 	return rc;
 }
@@ -2519,24 +2509,53 @@ cleanup:
 gboolean
 lasso_base64_decode(const char *from, char **buffer, int *buffer_len)
 {
-	size_t len = strlen(from);
-	int ret;
+	int fromlen = 0;
+	xmlChar *out = NULL;
+	xmlSecSize outlen = 0;
+	xmlSecSize decodedlen = 0;
+	int rc = TRUE;
+	int ret = 0;
 
-	/* base64 map 4 bytes to 3 */
-	len = len / 4 + (len % 4 ? 1 : 0);
-	len *= 3;
-	len += 1; /* zero byte */
-	*buffer = g_malloc0(len);
-
-	xmlSecErrorsDefaultCallbackEnableOutput(FALSE);
-	ret = xmlSecBase64Decode(BAD_CAST from, BAD_CAST *buffer, len);
-	xmlSecErrorsDefaultCallbackEnableOutput(TRUE);
-	if (ret <= 0) {
-		lasso_release_string(*buffer);
+	if (! from) {
 		return FALSE;
 	}
-	*buffer_len = ret;
-	return TRUE;
+
+	fromlen = strlen(from);
+
+	/* base64 map 4 bytes to 3 */
+	outlen = fromlen / 4 + (fromlen % 4 ? 1 : 0);
+	outlen *= 3;
+	outlen += 1; /* zero byte */
+	out = g_malloc0(outlen);
+
+#if LASSO_XMLSEC_VERSION_NUMBER >= 0x010223
+	xmlSecErrorsDefaultCallbackEnableOutput(FALSE);
+	ret = xmlSecBase64Decode_ex(BAD_CAST from, out, outlen, &decodedlen);
+	xmlSecErrorsDefaultCallbackEnableOutput(TRUE);
+	if (ret == 0) {
+		out[outlen - 1] = 0;
+		lasso_transfer_string(*buffer, *((char**)&out));
+		*buffer_len = decodedlen;
+	} else {
+		rc = FALSE;
+	}
+#else
+	xmlSecErrorsDefaultCallbackEnableOutput(FALSE);
+	ret = xmlSecBase64Decode(BAD_CAST from, out, outlen);
+	xmlSecErrorsDefaultCallbackEnableOutput(TRUE);
+
+	if (ret >= 0) {
+		out[outlen - 1] = 0;
+		lasso_transfer_string(*buffer, *((char**)&out));
+		*buffer_len = ret;
+	} else {
+		rc = FALSE;
+	}
+#endif
+	if (out) {
+		lasso_release_string(out);
+	}
+	return rc;
 }
 
 /**
@@ -2559,10 +2578,10 @@ lasso_xmlURIEscapeStr(const xmlChar *from, const xmlChar *list)
 	int ri;
 
 	if (list == NULL)
-		list = "";
+		list = (xmlChar*)"";
 
 	for (fp = from; *fp; fp++) {
-		if (isalnum(*fp) || strchr("._~-", *fp) || strchr(list, *fp))
+		if (isalnum(*fp) || strchr("._~-", *fp) || strchr((char*)list, *fp))
 			len++;
 		else
 			len += 3;
@@ -2572,7 +2591,7 @@ lasso_xmlURIEscapeStr(const xmlChar *from, const xmlChar *list)
 	ri = 0;
 
 	for (fp = from; *fp; fp++) {
-		if (isalnum(*fp) || strchr("._~-", *fp) || strchr(list, *fp)) {
+		if (isalnum(*fp) || strchr("._~-", *fp) || strchr((char*)list, *fp)) {
 			result[ri++] = *fp;
 		} else {
 			int msb = (*fp & 0xf0) >> 4;
@@ -2640,36 +2659,30 @@ lasso_xmlsec_load_private_key(const char *filename_or_buffer, const char *passwo
 
 gboolean
 lasso_get_base64_content(xmlNode *node, char **content, size_t *length) {
-	xmlChar *base64, *stripped_base64;
-	xmlChar *result;
-	int base64_length;
-	int rc = 0;
+	xmlChar *base64 = NULL;
+	xmlChar *stripped_base64 = NULL;
+	char *decoded = NULL;
+	int decoded_length = 0;
+	int rc = TRUE;
 
-	if (! node || ! content || ! length)
-		return FALSE;
+	goto_cleanup_if_fail_with_rc(node && content && length, FALSE);
 
 	base64 = xmlNodeGetContent(node);
-	if (! base64)
-		return FALSE;
-	stripped_base64 = base64;
+	goto_cleanup_if_fail_with_rc(base64, FALSE);
+
 	/* skip spaces */
+	stripped_base64 = base64;
 	while (*stripped_base64 && isspace(*stripped_base64))
 		stripped_base64++;
 
-	base64_length = strlen((char*)stripped_base64);
-	result = g_new(xmlChar, base64_length);
-	xmlSecErrorsDefaultCallbackEnableOutput(FALSE);
-	rc = xmlSecBase64Decode(stripped_base64, result, base64_length);
-	xmlSecErrorsDefaultCallbackEnableOutput(TRUE);
-	xmlFree(base64);
-	if (rc < 0) {
-		return FALSE;
-	} else {
-		*content = (char*)g_memdup(result, rc);
-		xmlFree(result);
-		*length = rc;
-		return TRUE;
-	}
+	goto_cleanup_if_fail_with_rc(lasso_base64_decode((char*)stripped_base64, &decoded, &decoded_length), FALSE);
+	lasso_transfer_string(*content, decoded);
+	*length = decoded_length;
+cleanup:
+	lasso_release_xml_string(base64);
+	lasso_release_string(decoded);
+	return rc;
+
 }
 
 xmlSecKeyPtr
@@ -3166,13 +3179,13 @@ static char*
 lasso_get_saml_message(xmlChar **query_fields) {
 	int i = 0;
 	char *enc = NULL;
-	char *message = NULL;
+	char *raw_message = NULL;
+	char *gziped_message = NULL;
+	int gziped_message_len = 0;
 	char *saml_message = NULL;
-	char *decoded_message = NULL;
+	size_t saml_message_len = 0;
 	xmlChar *field = NULL;
 	char *t = NULL;
-	int rc = 0;
-	int len = 0;
 
 	for (i=0; (field=query_fields[i]); i++) {
 		t = strchr((char*)field, '=');
@@ -3184,11 +3197,11 @@ lasso_get_saml_message(xmlChar **query_fields) {
 			continue;
 		}
 		if (strcmp((char*)field, LASSO_SAML2_FIELD_REQUEST) == 0 || strcmp((char*)field, LASSO_SAML2_FIELD_RESPONSE) == 0) {
-			message = t+1;
+			raw_message = t+1;
 			continue;
 		}
 	}
-	if (message == NULL) {
+	if (raw_message == NULL) {
 		return NULL;
 	}
 	if (enc && strcmp(enc, LASSO_SAML2_DEFLATE_ENCODING) != 0) {
@@ -3196,23 +3209,12 @@ lasso_get_saml_message(xmlChar **query_fields) {
 		debug("Unknown URL encoding: %64s", enc);
 		return NULL;
 	}
-	len = strlen(message);
-	decoded_message = g_malloc(len);
-	if (! is_base64(message)) {
-		debug("message is not base64");
-		goto cleanup;
-	}
-	rc = xmlSecBase64Decode((xmlChar*)message, (xmlChar*)decoded_message, len);
-	if (rc < 0) {
-		debug("could not decode redirect SAML message");
-		goto cleanup;
-	}
-	/* rc contains the length of the result */
-	saml_message = (char*)lasso_inflate((unsigned char*) decoded_message, rc);
+
+	goto_cleanup_if_fail(lasso_base64_decode(raw_message, &gziped_message, &gziped_message_len))
+	saml_message = (char*)lasso_inflate((unsigned char*)gziped_message, gziped_message_len, &saml_message_len);
 cleanup:
-	if (decoded_message) {
-		lasso_release(decoded_message);
-	}
+	lasso_release_string(gziped_message);
+
 	return saml_message;
 }
 
@@ -3228,6 +3230,7 @@ lasso_xmltextreader_from_message(const char *message, char **to_free) {
 	char *needle;
 	xmlChar **query_fields = NULL;
 	char *decoded_message = NULL;
+	int decoded_message_len = 0;
 	xmlTextReader *reader = NULL;
 
 	g_assert(to_free);
@@ -3243,22 +3246,17 @@ lasso_xmltextreader_from_message(const char *message, char **to_free) {
 			}
 			len = strlen(message);
 		} else { /* POST */
-			int rc = 0;
-
 			if (! is_base64(message)) {
 				debug("POST message is not base64");
 				goto cleanup;
 			}
-			decoded_message = g_malloc(len);
-			rc = xmlSecBase64Decode((xmlChar*)message, (xmlChar*)decoded_message, len);
-			if (rc < 0) {
+			if (! lasso_base64_decode(message, &decoded_message, &decoded_message_len)) {
 				debug("could not decode POST SAML message");
 				goto cleanup;
 			}
-			len = rc;
-			decoded_message[len] = '\0';
-			message = *to_free = decoded_message;
-			decoded_message = NULL;
+			message = decoded_message;
+			len = decoded_message_len;
+			lasso_transfer_string(*to_free, decoded_message);
 		}
 	}
 
@@ -3266,9 +3264,7 @@ lasso_xmltextreader_from_message(const char *message, char **to_free) {
 		reader = xmlReaderForMemory(message, len, "", NULL, XML_PARSE_NONET);
 
 cleanup:
-	if (query_fields)
-		lasso_release_array_of_xml_strings(query_fields);
-	if (decoded_message)
-		lasso_release_string(decoded_message);
+	lasso_release_array_of_xml_strings(query_fields);
+	lasso_release_string(decoded_message);
 	return reader;
 }
